#!/usr/bin/perl -w
#
# webSMS Perl module by leecher 2006
# This module enables you to send SMS-messages by using the .ini files of my websms Miranda-plugin
# 
#

package WebSMS;

require LWP::UserAgent;
require HTTP::Request;
use URI::Escape;
require HTTP::Cookies;
require Config::Tiny;

sub new
{
  my ($self, $proxy, $configdir, $sender) = @_;

  # change this if you want:
  my $cookiejar = "cookies.txt";
  $self->{ua} = MyAgent->new();
  $self->{ua}->cookie_jar(HTTP::Cookies->new(file => $cookiejar, autosave => 1, ignore_discard => 1));
  $self->proxy($proxy);
  $self->configdir($configdir);
  $self->sender($sender);
  $self->debug(0);
}

sub debug
{
  my $self = shift;
  $self->{debug} = shift;
}

sub proxy
{
  my $self = shift;
  my $proxy = shift;
  if ($proxy) { $self->{ua}->proxy(['http', 'ftp'] => $proxy); }
}

sub configdir
{
  my $self = shift;
  my $configdir = shift;
  if ($configdir) { $self->{configdir} = $configdir; }
}

# Set phone number of sender.
sub sender
{
  my $self = shift;
  my $sender = shift;
  if ($sender) { $self->{sender} = $sender; }
}

sub find_cookie {
  my ($self, $file, $searchkey) = @_;
  
  local(*FILE, $_);
  local $/ = "\n";  # make sure we got standard record separator
  open(FILE, $file) or return;
  my $magic = <FILE>;
  unless ($magic =~ /^\#LWP-Cookies-(\d+\.\d+)/)
  {
    warn "$file does not seem to contain cookies";
    return;
  }
  while (<FILE>)
  {
    next unless s/^Set-Cookie3:\s*//;
    chomp;
    my $cookie;
    for $cookie (split_header_words($_))
    {
      my($key,$val) = splice(@$cookie, 0, 2);
      if ($key eq $searchkey)
      {
	return $val;
      }
    }
  }
  close(FILE);
  return;
}

sub PrepareURL {
  my ($self, $poststring, $prefix, $number, $message, $count, $username, $password, $ref_customvars) = @_;
  
  #
  # Prepare poststring/URL
  #
  # $LRurl not yet supported
  my $varnum;
  my @customvars = @$ref_customvars;
  $poststring =~ s/\$Prefix/$prefix/g;
  $poststring =~ s/\$Number/$number/g;
  $poststring =~ s/\$Message/$message/g;
  $poststring =~ s/\$Count/$count/g;
  $poststring =~ s/\$User/$username/g;
  $poststring =~ s/\$Pass/$password/g;
  $poststring =~ s/\$Sender/$self->{sender}/g;
  $poststring =~ /\$Cookie[(.+?)]/ and $varnum=$1;
  while ($varnum)
  {
    my $cookie = $self->find_cookie ($cookiejar, $varnum);
    if ($cookie)
    {
      $poststring =~ s/\$Cookie[$varnum]/$cookie/g;
    }
    $varnum="";
    $poststring =~ /\$Cookie[(.+?)]/ and $varnum=$1;
  }
  $poststring =~ /\$CustomVar(\d+?)/ and $varnum=$1;
  while ($varnum)
  {
    if ($customvars[$varnum-1])
    {
      $poststring =~ s/\$CustomVar$varnum/$customvars[$varnum-1]/g;
    } else {
      print "Warning: CustomVar$1 hasn't been extracted yet.\n";
      $poststring =~ s/\$CustomVar$varnum//g;
      # optionally you can remove this
      print "Aborting.\n";
      return;
    }
    $varnum="";
    $poststring =~ /\$CustomVar(\d+?)/ and $varnum=$1;
  }
  return $poststring;
}

sub SendGateway {
  my ($self, $configfile, $prefix, $number, $message) = @_;

  my $Config = Config::Tiny->new();
  $Config = Config::Tiny->read($configfile);
  if (!$Config)
  {
    print "ERROR: Config file $configfile not found.\n";
    return -1;
  }
  {
    my $GatewayName = $Config->{Gateway}->{Name};
    print "Processing Gateway $GatewayName...\n";
  }
  my $count = $Config->{Gateway}->{MaxChars} - length("$message");
  if ($Config->{Gateway}->{Editme})
  {
    print "ERROR: You should first edit your config file parameters in $configfile.\n";
    return -1;
  }
  if ($count < 0)
  {
    $count = 0;
    print "INFO: Message will be truncated.\n";
    $message = substr($message, 0, $Config->{Gateway}->{MaxChars});
  }
  my $username = $Config->{Gateway}->{Username};
  my $password = $Config->{Gateway}->{Password};
  my $i;
  my $pfx_found = 0;
  for ($i=1; ; $i++)
  {
    my $value = "Prefix$i";
    my $ini_prefix = $Config->{Gateway}->{$value};
    last if not $ini_prefix;
    $ini_prefix =~ /(.+?);/;
    if ($1 eq $prefix)
    {
      $ini_prefix =~ /.+?;(.*)/;
      print "Your provider is $1.\n";
      $pfx_found = $i;
      last;
    }
  }
  if ($pfx_found == 0)
  {
    print "Provider $prefix isn't supported by this Gateway.\n";
    return -1;
  }


  #
  # Now it's time to process the requests...
  #
  my $res;
  my @customvars;
  my @keys;

  for ($i=1; ; $i++)
  {
    my $section = "Request$i";
    my $method = $Config->{$section}->{Type};
    last if not $method;
    my $URL = $Config->{$section}->{URL};
    if (!$URL)
    {
      print "ERROR: URL wasn't specified in request #$i. Aborting.\n";
      last;
    }
    my $pfx = $prefix;
    {
      my $pfx_val = "MapPrefix$pfx_found";
      my $map_pfx = $Config->{$section}->{$pfx_val};
      if ($map_pfx) {$pfx = $map_pfx;}
    }

    $URL = $self->PrepareURL($URL, $pfx, $number, $message, $count, $username, $password, \@customvars);
    if (!$URL) { return -1; }
    my $req = HTTP::Request->new($method => $URL);
    my $referer = $Config->{$section}->{Referer};
    if ($referer) { $req->header(Referer => $referer); }
    my $host = $Config->{$section}->{Host};
    if ($host) { $req->header(Host => $host); }
    my $useragent = $Config->{$section}->{UserAgent};
    if ($useragent) { $req->user_agent($useragent); } else { $req->user_agent("Mozilla/4.0"); }

    if ($method eq "POST")
    {
      my $poststring = $Config->{$section}->{PostString};
      if (!$poststring)
      {
        print "ERROR: Method POST but no poststring present in Request #$i. Aborting.\n";
	last;
      }
      $req->header(Connection => "keep-alive");
      $req->content_type("application/x-www-form-urlencoded");
      $poststring = $self->PrepareURL($poststring, $pfx, $number, $message, $count, $username, $password, \@customvars);
      if (!$poststring) { return -1; }
      foreach (@keys)
      {
	$poststring.=$_;
      }
      if ($self->{debug}>0) { print ("Poststring = $poststring\n"); }
      $req->content($poststring);
    }

    $res = $self->{ua}->request($req);
    if (!$res->is_success)
    {
      print "Error, request #$i failed: ".$res->message()."\nAborting.\n";
      if ($self->{debug}>0) { print $res->content; }
      return -1;
    }
    if ($self->{debug}>0) { print $res->content; }
    
    my $j;
    for ($j=$#customvars + 2; ; $j++)
    {
      my $value = "CustomVar$j";
      my $c_var = $Config->{$section}->{$value};
      last if not $c_var;
      # Is there a proper way to do this automatically?
      $c_var =~ s/\\/\\\\/g;
      $c_var =~ s/\./\\\./g;
      $c_var =~ s/\[/\\\[/g;
      $c_var =~ s/\]/\\\]/g;
      $c_var =~ s/\^/\\\^/g;
      $c_var =~ s/\?/\\\?/g;
      $c_var =~ s/\//\\\//g;
      $c_var =~ s/\*/\.\*\?/g;

      $varnum="";
      $res->content =~ m/$c_var/ and $varnum=$1;
      if ($varnum)
      {
        push (@customvars, uri_escape($varnum));
	if ($self->{debug}>0) { print "New customvar: $varnum\n"; }
      } else {
        print "CustomVar$j could not be found in output. Aborting.\n";
	if ($self->{debug}>0) { print $res->content; }
	return -1;
      }
    }
    for ($j=$#keys + 2; ; $j++)
    {
      my $value = "Key$j";
      my $key_var = $Config->{$section}->{$value};
      last if not $key_var;
      $varnum="";
      $res->content =~ / name=\"*$key_var.*?value=\"(.*?)\"/i and $varnum=$1;
      if ($varnum)
      {
        push (@keys, "&$key_var=".uri_escape($varnum));
      } else {
        print "Key$j cound not be found in output. Aborting.\n";
	if ($self->{debug}>0) { print $res->content; }
	return -1;
      }
    }
    {
      my $delay = $Config->{$section}->{Delay};
      if ($delay) { sleep ($delay/1000); }
    }
  }
  my $success = $Config->{Gateway}->{Success};
  my $succeeded = -1;
  if ($success)
  {
    $res->content =~ /$success/ and $succeeded=1;
    if ($succeeded == 1)
    {
      print "Sending SMS was successful.\n";
    } else {
      print "Sending SMS via this gateway failed.\n";
      if ($self->{debug}>0) { print $res->content; }
    }
  } else {
    print "Unsure if we succeeded. Assuming: yes\n";
    $succeeded = 1;
  }
  return $succeeded; 
}

sub Send
{
  my ($self, $prefix, $number, $message) = @_;
 
  my @configfiles = glob($self->{configdir}."*.ini");
  my $retval;
  foreach (@configfiles)
  {
    $retval = $self->SendGateway ($_, $prefix, $number, $message);
    print "\n";
    if ($retval == 1) {last;}
  }
  if ($retval == -1)
  {
    print "Sending to all gateways failed, sorry.\n";
 }
 return $retval;
}

